package com.isu.gaswellwatch.entity;

import lombok.AllArgsConstructor;
import lombok.Builder;
import lombok.Data;
import lombok.NoArgsConstructor;

import java.io.Serializable;

@Data
@Builder
@NoArgsConstructor
@AllArgsConstructor
public class Response<T> implements Serializable {
    private int code = ResponseCode.SUCCESS.code();
    private String message = ResponseCode.SUCCESS.message();
    private T data;

    public static <T> Response<T> succeed() {
        return Response.<T>builder().code(ResponseCode.SUCCESS.code()).message(ResponseCode.SUCCESS.message()).build();
    }

    public static <T> Response<T> succeed(String message) {
        return Response.<T>builder().code(ResponseCode.SUCCESS.code()).message(message).build();
    }

    public static <T> Response<T> succeed(T t) {
        return Response.<T>builder().code(ResponseCode.SUCCESS.code()).message(ResponseCode.SUCCESS.message()).data(t)
            .build();
    }

    public static <T> Response<T> succeed(String message, T t) {
        return Response.<T>builder().code(ResponseCode.SUCCESS.code()).message(message).data(t)
                .build();
    }

    public static <T> Response<T> succeed(ResponseCode responseCode) {
        return Response.<T>builder().code(responseCode.code()).message(responseCode.message()).build();
    }

    public static <T> Response<T> succeed(ResponseCode responseCode, T t) {
        return Response.<T>builder().code(responseCode.code()).message(responseCode.message()).data(t)
                .build();
    }

    public static <T> Response<T> failed() {
        return Response.<T>builder().code(ResponseCode.FAILURE.code()).message(ResponseCode.FAILURE.message()).build();
    }

    public static <T> Response<T> failed(String message) {
        return Response.<T>builder().code(ResponseCode.FAILURE.code()).message(message).build();
    }

    public static <T> Response<T> failed(int code, String message) {
        return Response.<T>builder().code(code).message(message).build();
    }

    public static <T> Response<T> failed(ResponseCode responseCode) {
        return Response.<T>builder().code(responseCode.code()).message(responseCode.message()).build();
    }

    public static <T> Response<T> failed(ResponseCode responseCode, T t) {
        return Response.<T>builder().code(responseCode.code()).message(responseCode.message()).data(t).build();
    }

    public static <T> Response<T> failed(T t) {
        return Response.<T>builder().code(ResponseCode.FAILURE.code()).message(ResponseCode.FAILURE.message()).data(t)
            .build();
    }

    public static <T> Response<T> define(int code, T t) {
        return Response.<T>builder().code(code).message(ResponseCode.SUCCESS.message()).data(t).build();
    }

    public static <T> Response<T> define(int code, String message, T t) {
        return Response.<T>builder().code(code).message(message).data(t).build();
    }

}
